@@ -10,7 +10,7 @@
    or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
-/* $Id: gdevdrop.c 12275 2011-03-10 17:31:38Z robin $ */
+/* $Id$ */
 /* Default and device-independent RasterOp algorithms */
 #include "memory_.h"
 #include "gx.h"
@@ -24,7 +24,9 @@
 #include "gxdevrop.h"
 #include "gxgetbit.h"
 #include "gdevmem.h"            /* for mem_default_strip_copy_rop prototype */
+#include "gdevmpla.h"
 #include "gdevmrop.h"
+#include "gxdevsop.h"
 
 /*
  * Define the maximum amount of space we are willing to allocate for a
@@ -96,6 +98,24 @@ gx_default_strip_copy_rop(gx_device * dev,
                           int x, int y, int width, int height,
                           int phase_x, int phase_y,
                           gs_logical_operation_t lop)
+{
+    return gx_default_strip_copy_rop2(dev, sdata, sourcex, sraster, id,
+                                      scolors, textures, tcolors,
+                                      x, y, width, height,
+                                      phase_x, phase_y, lop, 0);
+}
+
+int
+gx_default_strip_copy_rop2(gx_device * dev,
+                           const byte * sdata, int sourcex,
+                           uint sraster, gx_bitmap_id id,
+                           const gx_color_index * scolors,
+                           const gx_strip_bitmap * textures,
+                           const gx_color_index * tcolors,
+                           int x, int y, int width, int height,
+                           int phase_x, int phase_y,
+                           gs_logical_operation_t lop,
+                           uint planar_height)
 {
     int depth = dev->color_info.depth;
     gs_memory_t *mem = dev->memory;
@@ -108,6 +128,7 @@ gx_default_strip_copy_rop(gx_device * dev,
     int block_height;
     int code;
     int py;
+    int is_planar = 0;
 
 #ifdef DEBUG
     if (gs_debug_c('b'))
@@ -128,11 +149,33 @@ gx_default_strip_copy_rop(gx_device * dev,
     if (max_height == 0)
         max_height = 1;
     block_height = min(height, max_height);
+    if (planar_height > 0)
+        block_height = planar_height;
     gs_make_mem_device_with_copydevice(&pmdev, mdproto, mem, -1, dev);
     pmdev->width = width;
     pmdev->height = block_height;
     pmdev->bitmap_memory = mem;
     pmdev->color_info = dev->color_info;
+    if (dev_proc(dev, dev_spec_op)(dev, gxdso_is_native_planar, NULL, 0))
+    {
+        gx_render_plane_t planes[GX_DEVICE_COLOR_MAX_COMPONENTS];
+        int num_comp = dev->color_info.num_components;
+        int depth = dev->color_info.depth/num_comp;
+        int i;
+        for (i = 0; i < num_comp; i++)
+        {
+            planes[i].shift = depth * (num_comp - 1 - i);
+            planes[i].depth = depth;
+            planes[i].index = i;
+        }
+        /* RJW: This code, like most of ghostscripts planar support,
+         * will only work if every plane has the same depth. */
+        draster = bitmap_raster(width * planes[0].depth);
+        code = gdev_mem_set_planar(pmdev, num_comp, planes);
+        if (code < 0)
+            return code;
+        is_planar = 1;
+    }
     code = (*dev_proc(pmdev, open_device))((gx_device *)pmdev);
     pmdev->is_open = true; /* not sure why we need this, but we do. */
     if (code < 0)
@@ -171,16 +214,36 @@ gx_default_strip_copy_rop(gx_device * dev,
             if (code < 0)
                 return code;
         }
-        code = (*dev_proc(pmdev, strip_copy_rop))
-            ((gx_device *)pmdev,
-             sdata + (py - y) * sraster, sourcex, sraster,
-             gx_no_bitmap_id, scolors, textures, tcolors,
-             0, 0, width, block_height, phase_x + x, phase_y + py, lop);
+        if (planar_height == 0) {
+            code = (*dev_proc(pmdev, strip_copy_rop))
+                        ((gx_device *)pmdev,
+                         sdata + (py - y) * sraster, sourcex, sraster,
+                         gx_no_bitmap_id, scolors, textures, tcolors,
+                         0, 0, width, block_height,
+                         phase_x + x, phase_y + py,
+                         lop);
+        } else {
+            code = (*dev_proc(pmdev, strip_copy_rop2))
+                        ((gx_device *)pmdev,
+                         sdata + (py - y) * sraster, sourcex, sraster,
+                         gx_no_bitmap_id, scolors, textures, tcolors,
+                         0, 0, width, block_height,
+                         phase_x + x, phase_y + py,
+                         lop, planar_height);
+        }
         if (code < 0)
             break;
-        code = (*dev_proc(dev, copy_color))
-            (dev, scan_line_base(pmdev, 0), 0, draster, gx_no_bitmap_id,
-             x, py, width, block_height);
+        if (is_planar) {
+            code = (*dev_proc(dev, copy_planes))
+                            (dev, scan_line_base(pmdev, 0), 0,
+                             draster, gx_no_bitmap_id,
+                             x, py, width, block_height, block_height);
+        } else {
+            code = (*dev_proc(dev, copy_color))
+                            (dev, scan_line_base(pmdev, 0), 0,
+                             draster, gx_no_bitmap_id,
+                             x, py, width, block_height);
+        }
         if (code < 0)
             break;
     }
@@ -408,9 +471,11 @@ pack_planar_from_standard(gx_device_memory * dev, int y, int destx,
     const byte *sp = src;
     int x, plane;
 
-    if (pdepth == 1 && dev->color_info.num_components == 4)
-        return pack_planar_cmyk_1bit_from_standard(dev, y, destx, src, width,
-                                                   depth, src_depth);
+    if (pdepth == 1 && dev->color_info.num_components == 4) {
+        pack_planar_cmyk_1bit_from_standard(dev, y, destx, src, width,
+                                            depth, src_depth);
+        return;
+    }
 
     for (plane = 0; plane < dev->num_planes; plane++) {
         byte *dest = scan_line_base(dev, y + plane * dev->height);
@@ -455,6 +520,12 @@ pack_planar_from_standard(gx_device_memory * dev, int y, int destx,
                 *dp[3]++ = (byte)pixel;
                 shift = 0;
                 break;
+            case 24:
+                *dp[0]++ = (byte)(pixel >> 16);
+                *dp[1]++ = (byte)(pixel >> 8);
+                *dp[2]++ = (byte)pixel;
+                shift = 0;
+                break;
             case 16:
                 *dp[0]++ = (byte)(pixel >> 8);
                 *dp[1]++ = (byte)pixel;
@@ -462,18 +533,38 @@ pack_planar_from_standard(gx_device_memory * dev, int y, int destx,
                 break;
             default:            /* 1, 2, 4, 8 */
             {
-                int pshift = 8-pdepth;
                 int pmask = (1<<pdepth)-1;
-                if ((shift -= pdepth) >= 0) {
-                    for (plane = 0; plane < dev->num_planes; pshift+=8,plane++)
-                        buf[plane] += (byte)(((pixel>>pshift) & pmask)<<shift);
-                } else {
+
+#ifdef ORIGINAL_CODE_KEPT_FOR_REFERENCE
+                /* Original code, kept for reference. I believe this copies
+                 * bits in the wrong order (i.e. the 0th component comes from
+                 * the lowest bits in pixel, rather than the highest), and
+                 * gets them from the wrong place (8 bits apart rather than
+                 * pdepth), but as I have no examples that actually tickle
+                 * this code, currently, I don't want to throw it away. */
+                int pshift = 8-pdepth;
+#else
+                /* We have pdepth*num_planes bits in 'pixel'. We need to copy
+                 * them (topmost bits first) into the buffer, packing them at
+                 * shift position. */
+                int pshift = pdepth*(dev->num_planes-1);
+#endif
+                /* Can we fit another pdepth bits into our buffer? */
+                shift -= pdepth;
+                if (shift < 0) {
+                    /* No, so flush the buffer to the planes. */
                     for (plane = 0; plane < dev->num_planes; plane++)
                         *dp[plane]++ = buf[plane];
                     shift += 8;
-                    for (plane = 0; plane < dev->num_planes; pshift+=8,plane++)
-                        buf[plane] = (byte)(((pixel>>pshift) & pmask)<<shift);
                 }
+                /* Copy the next pdepth bits into each planes buffer */
+#ifdef ORIGINAL_CODE_KEPT_FOR_REFERENCE
+                for (plane = 0; plane < dev->num_planes; pshift+=8,plane++)
+                    buf[plane] += (byte)(((pixel>>pshift) & pmask)<<shift);
+#else
+                for (plane = 0; plane < dev->num_planes; pshift-=pdepth,plane++)
+                    buf[plane] += (byte)(((pixel>>pshift) & pmask)<<shift);
+#endif
                 break;
             }
         }
@@ -497,16 +588,32 @@ pack_planar_from_standard(gx_device_memory * dev, int y, int destx,
  * operation, pack_from_standard to convert them back to the device
  * representation, and copy_color to write the pixels back.
  */
+int
+mem_default_strip_copy_rop2(gx_device * dev,
+                            const byte * sdata, int sourcex,
+                            uint sraster, gx_bitmap_id id,
+                            const gx_color_index * scolors,
+                            const gx_strip_bitmap * textures,
+                            const gx_color_index * tcolors,
+                            int x, int y, int width, int height,
+                            int phase_x, int phase_y,
+                            gs_logical_operation_t lop,
+                            uint planar_height)
+{
+    dlprintf("mem_default_strip_copy_rop2 should never be called!\n");
+    return gs_error_Fatal;
+}
+
 int
 mem_default_strip_copy_rop(gx_device * dev,
-                          const byte * sdata, int sourcex,
-                          uint sraster, gx_bitmap_id id,
-                          const gx_color_index * scolors,
-                          const gx_strip_bitmap * textures,
-                          const gx_color_index * tcolors,
-                          int x, int y, int width, int height,
-                          int phase_x, int phase_y,
-                          gs_logical_operation_t lop)
+                           const byte * sdata, int sourcex,
+                           uint sraster, gx_bitmap_id id,
+                           const gx_color_index * scolors,
+                           const gx_strip_bitmap * textures,
+                           const gx_color_index * tcolors,
+                           int x, int y, int width, int height,
+                           int phase_x, int phase_y,
+                           gs_logical_operation_t lop)
 {
     int depth = dev->color_info.depth;
     int rop_depth = (gx_device_has_color(dev) ? 24 : 8);
@@ -515,6 +622,11 @@ mem_default_strip_copy_rop(gx_device * dev,
         GB_COLORS_NATIVE | GB_ALPHA_NONE | GB_DEPTH_ALL |
         GB_PACKING_CHUNKY | GB_RETURN_ALL | GB_ALIGN_STANDARD |
         GB_OFFSET_0 | GB_OFFSET_ANY | GB_RASTER_STANDARD;
+    const gx_bitmap_format_t no_expand_t_options =
+        GB_COLORS_NATIVE | GB_ALPHA_NONE | GB_DEPTH_ALL |
+        GB_RETURN_ALL | GB_ALIGN_STANDARD |
+        GB_OFFSET_0 | GB_OFFSET_ANY | GB_RASTER_STANDARD |
+        ((textures && textures->num_planes > 1) ? GB_PACKING_PLANAR : GB_PACKING_CHUNKY);
     const gx_bitmap_format_t expand_options =
         (rop_depth > 8 ? GB_COLORS_RGB : GB_COLORS_GRAY) |
         GB_ALPHA_NONE | GB_DEPTH_8 |
@@ -549,6 +661,7 @@ mem_default_strip_copy_rop(gx_device * dev,
     gs_get_bits_params_t bit_params;
     gs_get_bits_params_t expand_params;
     gs_get_bits_params_t no_expand_params;
+    gs_get_bits_params_t no_expand_t_params;
     int max_height;
     int block_height, loop_height;
     int code;
@@ -576,11 +689,16 @@ mem_default_strip_copy_rop(gx_device * dev,
 
     /* We know the device is a memory device, so we can store the
      * result directly into its scan lines, unless it is planar. */
-    pack = (tdev->num_planes <= 1 ?
-            ((dev_proc(dev, map_cmyk_color) == cmyk_1bit_map_cmyk_color &&
-              rop_depth == 24) ?
-              pack_cmyk_1bit_from_standard : pack_from_standard) :
-            pack_planar_from_standard);
+    if (tdev->num_planes <= 1) {
+        if ((rop_depth == 24) && (dev_proc(dev, dev_spec_op)(dev,
+                                      gxdso_is_std_cmyk_1bit, NULL, 0) > 0)) {
+            pack = pack_cmyk_1bit_from_standard;
+        } else {
+            pack = pack_from_standard;
+        }
+    } else {
+        pack = pack_planar_from_standard;
+    }
 #ifdef DEBUG
     if (gs_debug_c('b'))
         trace_copy_rop("mem_default_strip_copy_rop",
@@ -603,6 +721,7 @@ mem_default_strip_copy_rop(gx_device * dev,
     expand_s = scolors == 0 && uses_s;
     expand_t = tcolors == 0 && uses_t;
     no_expand_params.options = no_expand_options;
+    no_expand_t_params.options = no_expand_t_options;
     if (expand_t) {
         /*
          * We don't want to wrap around more than once in Y when
@@ -676,7 +795,7 @@ mem_default_strip_copy_rop(gx_device * dev,
             rect.q.y = py + loop_height;
             expand_params.data[0] = texture_row;
             gx_get_bits_copy(dev, 0, textures->rep_width, loop_height,
-                             &expand_params, &no_expand_params,
+                             &expand_params, &no_expand_t_params,
                              textures->data + rep_y * textures->raster,
                              textures->raster);
             /*
@@ -752,6 +871,7 @@ gx_default_copy_rop(gx_device * dev,
     else {
         *(gx_tile_bitmap *) & tiles = *texture;
         tiles.rep_shift = tiles.shift = 0;
+        tiles.num_planes = 1;
         textures = &tiles;
     }
     return (*dev_proc(dev, strip_copy_rop))
@@ -775,6 +895,7 @@ gx_copy_rop_unaligned(gx_device * dev,
     else {
         *(gx_tile_bitmap *) & tiles = *texture;
         tiles.rep_shift = tiles.shift = 0;
+        tiles.num_planes = 1;
         textures = &tiles;
     }
     return gx_strip_copy_rop_unaligned
