@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gsicc.c 12267 2011-03-10 00:58:08Z mvrhel $ */
+/* $Id$ */
 /* Implementation of the ICCBased color space family */
 
 #include "math_.h"
@@ -26,31 +26,18 @@
 #include "gzstate.h"
 #include "gsicc.h"
 #include "gsicc_cache.h"
-#include "gsicc_littlecms.h"
+#include "gsicc_cms.h"
 #include "gxdevice.h"
 
 #define SAVEICCPROFILE 0
 
-/*
- * Discard a gs_cie_icc_s structure. This requires that we call the
- * destructor for ICC profile, lookup, and file objects (which are
- * stored in "foreign" memory).
- *
- * No special action is taken with respect to the stream pointer; that is
- * the responsibility of the client.  */
-static void
-cie_icc_finalize(void * pvicc_info)
-{
-
-    }
-
 /*
  * Color space methods for ICCBased color spaces.
    ICC spaces are now considered to be concrete in that
    they always provide a mapping to a specified destination
    profile.  As such they will have their own remap functions.
    These will simply potentially implement the transfer function,
-   apply any alpha value, and or end up going through halftoning. 
+   apply any alpha value, and or end up going through halftoning.
    There will not be any heuristic remap of rgb to gray etc */
 
 static cs_proc_num_components(gx_num_components_ICC);
@@ -85,21 +72,19 @@ const gs_color_space_type gs_color_space_type_ICC = {
     gx_cspace_is_linear_ICC
 };
 
-
-
 static inline void
-gsicc_remap_fast(unsigned short *psrc, unsigned short *psrc_cm,
-		       gsicc_link_t *icc_link)
+gsicc_remap_fast(gx_device *dev, unsigned short *psrc, unsigned short *psrc_cm,
+                 gsicc_link_t *icc_link)
 {
-    gscms_transform_color(icc_link, psrc, psrc_cm, 2, NULL);
+    (icc_link->procs.map_color)(dev, icc_link, psrc, psrc_cm, 2);
 }
 
 /* ICC color mapping linearity check, a 2-points case. Check only the 1/2 point */
 static int
 gx_icc_is_linear_in_line(const gs_color_space *cs, const gs_imager_state * pis,
-		gx_device *dev, 
-		const gs_client_color *c0, const gs_client_color *c1,
-		float smoothness, gsicc_link_t *icclink)
+                        gx_device *dev,
+                        const gs_client_color *c0, const gs_client_color *c1,
+                        float smoothness, gsicc_link_t *icclink)
 {
     int nsrc = cs->type->num_components(cs);
     int ndes = dev->color_info.num_components;
@@ -120,9 +105,9 @@ gx_icc_is_linear_in_line(const gs_color_space *cs, const gs_imager_state * pis,
         src01[k] = ((unsigned int) src0[k] + (unsigned int) src1[k]) >> 1;
     }
     /* Transform the end points and the interpolated point */
-    gsicc_remap_fast(&(src0[0]), &(des0[0]), icclink);
-    gsicc_remap_fast(&(src1[0]), &(des1[0]), icclink);
-    gsicc_remap_fast(&(src01[0]), &(des01[0]), icclink);
+    gsicc_remap_fast(dev, &(src0[0]), &(des0[0]), icclink);
+    gsicc_remap_fast(dev, &(src1[0]), &(des1[0]), icclink);
+    gsicc_remap_fast(dev, &(src01[0]), &(des01[0]), icclink);
     /* Interpolate 1/2 value in des space and compare */
     for (k = 0; k < ndes; k++) {
         interp_des = (des0[k] + des1[k]) >> 1;
@@ -135,9 +120,9 @@ gx_icc_is_linear_in_line(const gs_color_space *cs, const gs_imager_state * pis,
 /* Default icc color mapping linearity check, a triangle case. */
 static int
 gx_icc_is_linear_in_triangle(const gs_color_space *cs, const gs_imager_state * pis,
-		gx_device *dev, 
-		const gs_client_color *c0, const gs_client_color *c1,
-		const gs_client_color *c2, float smoothness, gsicc_link_t *icclink)
+                gx_device *dev,
+                const gs_client_color *c0, const gs_client_color *c1,
+                const gs_client_color *c2, float smoothness, gsicc_link_t *icclink)
 {
     /* Check 4 points middle points of 3 sides and middle of one side with
        other point.  We avoid divisions this way. */
@@ -167,18 +152,18 @@ gx_icc_is_linear_in_triangle(const gs_color_space *cs, const gs_imager_state * p
         src1[k] = (unsigned short) (c1->paint.values[k]*65535);
         src2[k] = (unsigned short) (c2->paint.values[k]*65535);
         src01[k] = (src0[k] + src1[k]) >> 1;
-        src02[k] = (src0[k] + src2[k]) >> 1;   
+        src02[k] = (src0[k] + src2[k]) >> 1;
         src12[k] = (src1[k] + src2[k]) >> 1;
-        src012[k] = (src12[k] + src0[k]) >> 1;  
+        src012[k] = (src12[k] + src0[k]) >> 1;
     }
     /* Map the points */
-    gsicc_remap_fast(&(src0[0]), &(des0[0]), icclink);
-    gsicc_remap_fast(&(src1[0]), &(des1[0]), icclink);
-    gsicc_remap_fast(&(src2[0]), &(des2[0]), icclink);
-    gsicc_remap_fast(&(src01[0]), &(des01[0]), icclink);
-    gsicc_remap_fast(&(src12[0]), &(des12[0]), icclink);
-    gsicc_remap_fast(&(src02[0]), &(des02[0]), icclink);
-    gsicc_remap_fast(&(src012[0]), &(des012[0]), icclink);
+    gsicc_remap_fast(dev, &(src0[0]), &(des0[0]), icclink);
+    gsicc_remap_fast(dev, &(src1[0]), &(des1[0]), icclink);
+    gsicc_remap_fast(dev, &(src2[0]), &(des2[0]), icclink);
+    gsicc_remap_fast(dev, &(src01[0]), &(des01[0]), icclink);
+    gsicc_remap_fast(dev, &(src12[0]), &(des12[0]), icclink);
+    gsicc_remap_fast(dev, &(src02[0]), &(des02[0]), icclink);
+    gsicc_remap_fast(dev, &(src012[0]), &(des012[0]), icclink);
     /* Interpolate in des space and check it */
     for (k = 0; k < ndes; k++){
         interp_des = (des0[k] + des1[k]) >> 1;
@@ -201,31 +186,31 @@ gx_icc_is_linear_in_triangle(const gs_color_space *cs, const gs_imager_state * p
 /* ICC color mapping linearity check. */
 int
 gx_cspace_is_linear_ICC(const gs_color_space *cs, const gs_imager_state * pis,
-		gx_device *dev, 
-		const gs_client_color *c0, const gs_client_color *c1,
-		const gs_client_color *c2, const gs_client_color *c3,
-		float smoothness, gsicc_link_t *icclink)
+                gx_device *dev,
+                const gs_client_color *c0, const gs_client_color *c1,
+                const gs_client_color *c2, const gs_client_color *c3,
+                float smoothness, gsicc_link_t *icclink)
 {
     /* Assuming 2 <= nc <= 4. We don't need other cases. */
     /* With nc == 4 assuming a convex plain quadrangle in the client color space. */
     int code;
-    
+
     /* Do a quick check if we are in a halftone situation. If yes,
        then we should not be doing this linear check */
     if (gx_device_must_halftone(dev)) return 0;
     if (icclink->is_identity) return 1; /* Transform is identity, linear! */
 
     if (dev->color_info.separable_and_linear != GX_CINFO_SEP_LIN)
-	return_error(gs_error_rangecheck);
+        return_error(gs_error_rangecheck);
     if (c2 == NULL)
-	return gx_icc_is_linear_in_line(cs, pis, dev, c0, c1, smoothness, icclink);
-    code = gx_icc_is_linear_in_triangle(cs, pis, dev, c0, c1, c2, 
+        return gx_icc_is_linear_in_line(cs, pis, dev, c0, c1, smoothness, icclink);
+    code = gx_icc_is_linear_in_triangle(cs, pis, dev, c0, c1, c2,
                                             smoothness, icclink);
     if (code <= 0)
-	return code;
+        return code;
     if (c3 == NULL)
-	return 1;
-    return gx_icc_is_linear_in_triangle(cs, pis, dev, c1, c2, c3, 
+        return 1;
+    return gx_icc_is_linear_in_triangle(cs, pis, dev, c1, c2, c3,
                                                 smoothness, icclink);
 }
 /*
@@ -247,7 +232,7 @@ gx_init_ICC(gs_client_color * pcc, const gs_color_space * pcs)
     int     i, ncomps = pcs->cmm_icc_profile_data->num_comps;
 
     for (i = 0; i < ncomps; ++i)
-	pcc->paint.values[i] = 0.0;
+        pcc->paint.values[i] = 0.0;
 
     /* make sure that [ 0, ... 0] is in range */
     gx_restrict_ICC(pcc, pcs);
@@ -273,22 +258,24 @@ gx_restrict_ICC(gs_client_color * pcc, const gs_color_space * pcs)
     }
 }
 
-/* If the color is already concretized, then we are in the color space 
+/* If the color is already concretized, then we are in the color space
    defined by the device profile.  The remaining things to do would
    be to potentially apply alpha, apply the transfer function, and
    do any halftoning.  The remap is based upon the ICC profile defined
    in the device profile entry of the profile manager. */
 int
 gx_remap_concrete_ICC(const frac * pconc, const gs_color_space * pcs,
-	gx_device_color * pdc, const gs_imager_state * pis, gx_device * dev,
-			  gs_color_select_t select)
+        gx_device_color * pdc, const gs_imager_state * pis, gx_device * dev,
+                          gs_color_select_t select)
 {
-    const cmm_profile_t *icc_profile = dev->device_icc_profile;
-    int num_colorants = icc_profile->num_comps;
+    int num_colorants;
     int code;
+    cmm_dev_profile_t *dev_profile;
 
+    code = dev_proc(dev, get_profile)(dev, &dev_profile);
+    num_colorants = gsicc_get_device_profile_comps(dev_profile);
     switch( num_colorants ) {
-        case 1: 
+        case 1:
             code = gx_remap_concrete_DGray(pconc, pcs, pdc, pis, dev, select);
             break;
         case 3:
@@ -303,7 +290,7 @@ gx_remap_concrete_ICC(const frac * pconc, const gs_color_space * pcs,
             code = -1;
             break;
         }
-    return(code);
+    return code;
     }
 
 /*
@@ -311,8 +298,8 @@ gx_remap_concrete_ICC(const frac * pconc, const gs_color_space * pcs,
  */
 int
 gx_remap_ICC(const gs_client_color * pcc, const gs_color_space * pcs,
-	gx_device_color * pdc, const gs_imager_state * pis, gx_device * dev,
-		gs_color_select_t select)
+        gx_device_color * pdc, const gs_imager_state * pis, gx_device * dev,
+                gs_color_select_t select)
 {
     gsicc_link_t *icc_link;
     gsicc_rendering_param_t rendering_params;
@@ -322,11 +309,17 @@ gx_remap_ICC(const gs_client_color * pcc, const gs_color_space * pcs,
     int k,i;
 #ifdef DEBUG
     int num_src_comps;
-    int num_des_comps;
 #endif
+    int num_des_comps;
+    int code;
+    cmm_dev_profile_t *dev_profile;
 
+    code = dev_proc(dev, get_profile)(dev, &dev_profile);
+    num_des_comps = gsicc_get_device_profile_comps(dev_profile);
     rendering_params.black_point_comp = BP_ON;
-    rendering_params.object_type = GS_PATH_TAG;
+    rendering_params.graphics_type_tag = dev->graphics_type_tag;
+    /* Need to figure out which one rules here on rendering intent.  The
+       source of the device */
     rendering_params.rendering_intent = pis->renderingintent;
 
     /* Need to clear out psrc_cm in case we have separation bands that are
@@ -344,28 +337,30 @@ gx_remap_ICC(const gs_client_color * pcc, const gs_color_space * pcs,
         }
     }
     /* Get a link from the cache, or create if it is not there. Need to get 16 bit profile */
-    icc_link = gsicc_get_link(pis, dev, pcs, NULL, &rendering_params, pis->memory, false);
+    icc_link = gsicc_get_link(pis, dev, pcs, NULL, &rendering_params, pis->memory);
+    if (icc_link == NULL) {
+        return gs_rethrow(-1, "Could not create ICC link:  Check profiles");
+    }
     if (icc_link->is_identity) {
         psrc_temp = &(psrc[0]);
     } else {
         /* Transform the color */
         psrc_temp = &(psrc_cm[0]);
-        gscms_transform_color(icc_link, psrc, psrc_temp, 2, NULL);
+        (icc_link->procs.map_color)(dev, icc_link, psrc, psrc_temp, 2);
     }
 #ifdef DEBUG
     if (!icc_link->is_identity) {
         num_src_comps = pcs->cmm_icc_profile_data->num_comps;
-        num_des_comps = dev->device_icc_profile->num_comps;
-        if_debug0('c',"[c]ICC remap [ ");
+        if_debug0(gs_debug_flag_icc,"[icc] remap [ ");
         for (k = 0; k < num_src_comps; k++) {
-            if_debug1('c', "%d ",psrc[k]);
-}
-        if_debug0('c',"] --> [ ");
+            if_debug1(gs_debug_flag_icc, "%d ",psrc[k]);
+        }
+        if_debug0(gs_debug_flag_icc,"] --> [ ");
         for (k = 0; k < num_des_comps; k++) {
-            if_debug1('c', "%d ",psrc_temp[k]);
-}
-        if_debug0('c',"]\n");
-}
+            if_debug1(gs_debug_flag_icc, "%d ",psrc_temp[k]);
+        }
+        if_debug0(gs_debug_flag_icc,"]\n");
+    }
 #endif
     /* Release the link */
     gsicc_release_link(icc_link);
@@ -373,7 +368,7 @@ gx_remap_ICC(const gs_client_color * pcc, const gs_color_space * pcs,
        the transfer function and potentially the halftoning */
     /* Right now we need to go from unsigned short to frac.  I really
        would like to avoid this sort of stuff.  That will come. */
-    for ( k = 0; k< dev->device_icc_profile->num_comps; k++){
+    for ( k = 0; k < num_des_comps; k++){
         conc[k] = ushort2frac(psrc_temp[k]);
     }
     gx_remap_concrete_ICC(conc, pcs, pdc, pis, dev, select);
@@ -381,7 +376,7 @@ gx_remap_ICC(const gs_client_color * pcc, const gs_color_space * pcs,
     /* Save original color space and color info into dev color */
     i = pcs->cmm_icc_profile_data->num_comps;
     for (i--; i >= 0; i--)
-	pdc->ccolor.paint.values[i] = pcc->paint.values[i];
+        pdc->ccolor.paint.values[i] = pcc->paint.values[i];
     pdc->ccolor_valid = true;
     return 0;
 }
@@ -403,26 +398,34 @@ gx_concretize_ICC(
     unsigned short psrc[GS_CLIENT_COLOR_MAX_COMPONENTS], psrc_cm[GS_CLIENT_COLOR_MAX_COMPONENTS];
     int k;
     unsigned short *psrc_temp;
+    int num_des_comps;
+    int code;
+    cmm_dev_profile_t *dev_profile;
 
+    code = dev_proc(dev, get_profile)(dev, &dev_profile);
+    num_des_comps = gsicc_get_device_profile_comps(dev_profile);
     /* Define the rendering intents.  MJV to fix */
     rendering_params.black_point_comp = BP_ON;
-    rendering_params.object_type = GS_PATH_TAG;
+    rendering_params.graphics_type_tag = dev->graphics_type_tag;
     rendering_params.rendering_intent = pis->renderingintent;
-    for (k = 0; k < pcs->cmm_icc_profile_data->num_comps; k++){
+    for (k = 0; k < pcs->cmm_icc_profile_data->num_comps; k++) {
         psrc[k] = (unsigned short) (pcc->paint.values[k]*65535.0);
-	}
+    }
     /* Get a link from the cache, or create if it is not there. Get 16 bit profile */
-    icc_link = gsicc_get_link(pis, dev, pcs, NULL, &rendering_params, pis->memory, false);
+    icc_link = gsicc_get_link(pis, dev, pcs, NULL, &rendering_params, pis->memory);
+    if (icc_link == NULL) {
+        return gs_rethrow(-1, "Could not create ICC link:  Check profiles");
+    }
     /* Transform the color */
     if (icc_link->is_identity) {
         psrc_temp = &(psrc[0]);
     } else {
         /* Transform the color */
         psrc_temp = &(psrc_cm[0]);
-        gscms_transform_color(icc_link, psrc, psrc_temp, 2, NULL);
+        (icc_link->procs.map_color)(dev, icc_link, psrc, psrc_temp, 2);
     }
     /* This needs to be optimized */
-    for (k = 0; k < dev->device_icc_profile->num_comps; k++){
+    for (k = 0; k < num_des_comps; k++){
         pconc[k] = float2frac(((float) psrc_temp[k])/65535.0);
     }
     /* Release the link */
@@ -430,21 +433,20 @@ gx_concretize_ICC(
     return 0;
 }
 
-	/*
+        /*
  * Finalize the contents of an ICC color space. Now that color space
  * objects have straightforward reference counting discipline, there's
  * nothing special about it. In the previous state of affairs, the
  * argument in favor of correct reference counting spoke of "an
  * unintuitive but otherwise legitimate state of affairs".
-	 */
+         */
 static void
 gx_final_ICC(const gs_color_space * pcs)
 {
-  /*  if (pcs->cmm_icc_profile_data != NULL) {
+    if (pcs->cmm_icc_profile_data != NULL) {
         rc_decrement_only(pcs->cmm_icc_profile_data, "gx_final_ICC");
-    } */
-    /* rc_decrement_only(pcs->params.icc.picc_info, "gx_final_ICC"); */
     }
+}
 
 /*
  * Install an ICCBased color space.
@@ -456,8 +458,8 @@ static int
 gx_install_ICC(gs_color_space * pcs, gs_state * pgs)
 {
     /* update the stub information used by the joint caches */
-		return 0;
-        }
+    return 0;
+}
 
 /*
  * Constructor for ICCBased color space. As with the other color space
@@ -485,7 +487,7 @@ gx_serialize_ICC(const gs_color_space * pcs, stream * s)
     int code = gx_serialize_cspace_type(pcs, s);
 
     if (code < 0)
-	return code;
+        return code;
     profile__serial = (gsicc_serialized_profile_t*) pcs->cmm_icc_profile_data;
     code = sputs(s, (byte *)profile__serial, sizeof(gsicc_serialized_profile_t), &n);
     return(code);
@@ -509,3 +511,9 @@ gx_set_overprint_ICC(const gs_color_space * pcs, gs_state * pgs)
     return gx_set_overprint_cmyk(pcs, pgs);
 }
 
+int
+gx_default_get_profile(gx_device *dev, cmm_dev_profile_t **profile) 
+{
+    *profile = dev->icc_struct;
+    return 0;
+}
