@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gsstate.c 11552 2010-07-29 14:29:19Z robin $ */
+/* $Id$ */
 /* Miscellaneous graphics state operators for Ghostscript library */
 #include "gx.h"
 #include "memory_.h"
@@ -471,7 +471,6 @@ gs_grestoreall_for_restore(gs_state * pgs, gs_state * saved)
     return gs_grestore(pgs);
 }
 
-
 /* Restore to the bottommost graphics state (at this save level). */
 int
 gs_grestoreall(gs_state * pgs)
@@ -853,7 +852,8 @@ gstate_free_parts(const gs_state * parts, gs_memory_t * mem, client_name_t cname
     if (!parts->effective_clip_shared)
         gx_cpath_free(parts->effective_clip_path, cname);
     gx_cpath_free(parts->clip_path, cname);
-    gx_path_free(parts->path, cname);
+    if (parts->path)
+	gx_path_free(parts->path, cname);
 }
 
 /* Allocate the privately allocated parts of a gstate. */
@@ -997,7 +997,6 @@ gstate_clone(gs_state * pfrom, gs_memory_t * mem, client_name_t cname,
     return 0;
 }
 
-
 /* Adjust reference counters for the whole clip stack */
 /* accessible from the given point */
 static void
@@ -1031,7 +1030,6 @@ gstate_free_contents(gs_state * pgs)
         (*pgs->client_procs.free) (pgs->client_data, mem);
     gs_free_object(mem, pgs->line_params.dash.pattern, cname);
     gstate_free_parts(pgs, mem, cname);
-    gs_imager_state_release((gs_imager_state *)pgs);
 }
 
 /* Copy one gstate to another. */
