@@ -1,6 +1,6 @@
 /* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
-  
+
    This software is provided AS-IS with no warranty, either express or
    implied.
 
@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gxpaint.c 10940 2010-03-18 00:57:10Z robin $ */
+/* $Id$ */
 /* Graphics-state-aware fill and stroke procedures */
 #include "gx.h"
 #include "gzstate.h"
@@ -24,15 +24,17 @@
 static bool caching_an_outline_font(const gs_state * pgs)
 {
     return pgs->in_cachedevice > 1 &&
-	    pgs->font != NULL &&
-	    pgs->font->FontType != ft_user_defined && 
-	    pgs->font->FontType != ft_CID_user_defined;
+            pgs->font != NULL &&
+            pgs->font->FontType != ft_user_defined &&
+            pgs->font->FontType != ft_PCL_user_defined &&
+            pgs->font->FontType != ft_GL2_stick_user_defined &&
+            pgs->font->FontType != ft_CID_user_defined;
 }
 
 /* Fill a path. */
 int
 gx_fill_path(gx_path * ppath, gx_device_color * pdevc, gs_state * pgs,
-	     int rule, fixed adjust_x, fixed adjust_y)
+             int rule, fixed adjust_x, fixed adjust_y)
 {
     gx_device *dev = gs_currentdevice_inline(pgs);
     gx_clip_path *pcpath;
@@ -40,13 +42,13 @@ gx_fill_path(gx_path * ppath, gx_device_color * pdevc, gs_state * pgs,
     gx_fill_params params;
 
     if (code < 0)
-	return code;
+        return code;
     params.rule = rule;
     params.adjust.x = adjust_x;
     params.adjust.y = adjust_y;
     params.flatness = (caching_an_outline_font(pgs) ? 0.0 : pgs->flatness);
     return (*dev_proc(dev, fill_path))
-	(dev, (const gs_imager_state *)pgs, ppath, &params, pdevc, pcpath);
+        (dev, (const gs_imager_state *)pgs, ppath, &params, pdevc, pcpath);
 }
 
 /* Stroke a path for drawing or saving. */
@@ -59,35 +61,35 @@ gx_stroke_fill(gx_path * ppath, gs_state * pgs)
     gx_stroke_params params;
 
     if (code < 0)
-	return code;
+        return code;
     params.flatness = (caching_an_outline_font(pgs) ? 0.0 : pgs->flatness);
     params.traditional = false;
     return (*dev_proc(dev, stroke_path))
-	(dev, (const gs_imager_state *)pgs, ppath, &params,
-	 gs_currentdevicecolor_inline(pgs), pcpath);
+        (dev, (const gs_imager_state *)pgs, ppath, &params,
+         gs_currentdevicecolor_inline(pgs), pcpath);
 }
 
 int
 gx_stroke_add(gx_path * ppath, gx_path * to_path,
-	      const gs_state * pgs, bool traditional)
+              const gs_state * pgs, bool traditional)
 {
     gx_stroke_params params;
 
     params.flatness = (caching_an_outline_font(pgs) ? 0.0 : pgs->flatness);
     params.traditional = traditional;
     return gx_stroke_path_only(ppath, to_path, pgs->device,
-			       (const gs_imager_state *)pgs,
-			       &params, NULL, NULL);
+                               (const gs_imager_state *)pgs,
+                               &params, NULL, NULL);
 }
 
 int
 gx_imager_stroke_add(gx_path *ppath, gx_path *to_path,
-		     gx_device *dev, const gs_imager_state *pis)
+                     gx_device *dev, const gs_imager_state *pis)
 {
     gx_stroke_params params;
 
     params.flatness = pis->flatness;
     params.traditional = false;
     return gx_stroke_path_only(ppath, to_path, dev, pis,
-			       &params, NULL, NULL);
+                               &params, NULL, NULL);
 }
