@@ -221,7 +221,11 @@ typedef void *LPVOID;
 
 // The specification for "inline" is section 6.7.4 of the C99 standard (ISO/IEC 9899:1999). 
 
+#ifdef __HP_cc
+#define LCMS_INLINE static __inline
+#else
 #define LCMS_INLINE static inline
+#endif
 
 #else
 
@@ -1492,8 +1496,8 @@ Fixed32 cdecl FixedSquare(Fixed32 a);
 
 #ifdef USE_INLINE
 
-LCMS_INLINE Fixed32 ToFixedDomain(int a)        { return a + ((a + 0x7fff) / 0xffff); }
-LCMS_INLINE int     FromFixedDomain(Fixed32 a)  { return a - ((a + 0x7fff) >> 16); }   
+#define ToFixedDomain(a)    ( (a) + (((a) + 0x7fff) / 0xffff) )
+#define FromFixedDomain(a)  ( (a) - (((a) + 0x7fff) >> 16) )  
 
 #else
 
@@ -2056,6 +2060,8 @@ LPGAMMATABLE _cmsBuildKToneCurve(cmsHTRANSFORM hCMYK2CMYK, int nPoints);
 // Validates a LUT
 LCMSBOOL cdecl _cmsValidateLUT(LPLUT NewLUT);
 
+// Choose the transform function to use
+void _cmsSetPrecalculatedTransform(_LPcmsTRANSFORM p);
 
 // These are two VITAL macros, from converting between 8 and 16 bit
 // representation. 
