@@ -1,6 +1,6 @@
 /* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
-  
+
    This software is provided AS-IS with no warranty, either express or
    implied.
 
@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: ifrpred.h 9043 2008-08-28 22:48:19Z giles $ */
+/* $Id$ */
 /* filter_read_predictor prototype */
 
 #ifndef ifrpred_INCLUDED
@@ -19,7 +19,7 @@
 
 /* Exported by zfdecode.c for zfzlib.c */
 int filter_read_predictor(i_ctx_t *i_ctx_p, int npop,
-			  const stream_template * template,
-			  stream_state * st);
+                          const stream_template * templat,
+                          stream_state * st);
 
 #endif /* ifrpred_INCLUDED */
